"""
Test if one domain is significantly rewried in another biological condition
Usage:
    scriptname dvsd one <chr> <start> <end> <hic0> <hic1> [options]
    scriptname dvsd multiple <hic0> <hic1> <bed> [options]
    scriptname visualization <chr> <start> <end> <hic0> <hic1> [options]
    scriptname adjustment <method> <input> <output> [options]

Options:
    --ofile filepath for output file  [default: stdout]
    --oprefix prefix for output files
    --oprefixFig prefix for output figures
    --sep deliminator for hicfile  [default: \t]
    --hicnorm hic matrix normalization method  [default: KR]
    --chrn chromosome number  [default: ALL]
    --reso resolution for hicfile  [default: 100000]
    --ncore number of parallel process  [default: 10]
    --min_nbin effective number of bin  [default: 10]
    --f parameters for filtering the null values of the matrix[0~1)  [default: 0.5]
    --filter As long as the pvalue of TADs is less than 0.05 after adjustment if argument is true  [default: false]


"""

import matplotlib
matplotlib.use('Agg')
import os
import sys
import pandas as pd
import numpy as np
from docopt import docopt
from  multiprocessing import Pool
from utils import comp2domins_by_twtest, loadtads, visualization

opts = __doc__
scriptname = os.path.basename(__file__)
print(scriptname)
opts = opts.replace("scriptname", scriptname)
opts = docopt(doc=opts)
# save opts to output for every output
opts_df = pd.DataFrame.from_dict(opts, orient='index')
opts_df.index = '#' + opts_df.index

if(opts['dvsd']):
    if opts['one']:
        result = comp2domins_by_twtest(chrn=opts['<chr>'], start=int(opts['<start>']), end=int(opts['<end>']), reso=int(opts['--reso']), hicnorm=opts['--hicnorm'], fhic0=opts['<hic0>'], fhic1=opts['<hic1>'], min_nbin=int(opts['--min_nbin']), f=opts['--f'])
        result = pd.DataFrame(result)
        print(result)
        opts_df.to_csv(opts['--ofile'], sep='\t', index=True, header=False)
        result.to_csv(opts['--ofile'], sep='\t', index=False, header=False, mode='a')

    if opts['multiple']:
        reso = int(opts['--reso'])
        tadb = loadtads(opts['<bed>'], sep=opts['--sep'], chrnum=opts['--chrn'], min_nbin=int(opts['--min_nbin']), reso=int(opts['--reso']))
        # Parallel implementation
        def comp2domins_by_twtest_parallel(i):
            tmp_res = comp2domins_by_twtest(chrn=tadb.iloc[i, 0], start=tadb.iloc[i, 1], 
            end=tadb.iloc[i, 2], reso=int(opts['--reso']), hicnorm=opts['--hicnorm'],
             fhic0=opts['<hic0>'], fhic1=opts['<hic1>'],min_nbin=int(opts['--min_nbin']),f=opts['--f'])
            return tmp_res
        if __name__=='__main__':
            P = Pool(int(opts['--ncore']))
            note = []
            for i in range(tadb.shape[0]):
                note.append(P.apply_async(comp2domins_by_twtest_parallel, [i]))
            P.close()
            P.join()
            result = []
            for i in note:
                result.append(i.get())
            result = pd.DataFrame(result)

            # first the options; then the result
            opts_df.to_csv(opts['--ofile'], sep='\t', index=True, header=False)
            result.to_csv(opts['--ofile'], sep='\t', index=False, header=False, mode='a')

elif(opts['visualization']):
    #
    def save_fig(fig_id, tight_layout=True, fig_extension="pdf", resolution=600):
        path = os.path.join(fig_id + "." + fig_extension)
        print("Saving figure", fig_id)
        if tight_layout:
            plt.tight_layout()
            plt.savefig(path, format=fig_extension, dpi=resolution)

    def plot(mat,outputfile):
        fig,ax = plt.subplots(figsize=(6,6))
        sns.heatmap(data=mat,
                vmax=int(round(np.nanmax(mat)))/5,
                cmap=cdict,cbar=False,
                mask=mat<0.1,square=True)
        plt.xticks([])
        plt.yticks([])
        ax.tick_params(bottom=False, top=False, left=False,right=False)
        save_fig(outputfile)

    import matplotlib.pyplot as plt
    import seaborn as sns
    from matplotlib.colors import LinearSegmentedColormap
    cdict=LinearSegmentedColormap.from_list('mycmap',['w','#FF0000'])
    result = visualization(chrn=opts['<chr>'], start=int(opts['<start>']),
                           end=int(opts['<end>']), reso=int(opts['--reso']),
                           hicnorm=opts['--hicnorm'], fhic0=opts['<hic0>'],
                           fhic1=opts['<hic1>'])
    #the lower triangle is the first input hic0, the upper triangle is the second input hic1
    matrix1 = result[0] 
    matrix2 = result[1]
    matrix1[np.isnan(matrix1)] = 1
    matrix2[np.isnan(matrix2)] = 1
    tril_mat = (np.tril(matrix1,-1))
    triu_mat = (np.triu(matrix2,1))
    mat = tril_mat + triu_mat
    plot(mat,opts['--ofile'])


elif(opts['adjustment']):
    import statsmodels.stats.multitest as smm
    def read_compared(inputfile, method='fdr_bh', alpha=0.05, skiprows=25):
        df = pd.read_csv(inputfile, header=None, sep='\t', skiprows=skiprows, error_bad_lines=False)
        df.columns=['chr','start','end','region','stat','pvalue','bins']
        pvalue = df['pvalue']
        pvalue = df['pvalue'].fillna(pvalue.median())
        rej, pval_corr = smm.multipletests(pvalue, is_sorted=False, alpha=alpha, method=method)[:2]
        df['adj_pvalue'] = pval_corr
        return df


    res = read_compared(inputfile=opts['<input>'], method=opts['<method>'])
    if opts['--filter'] == "true":
        #save result
        res = res[res['adj_pvalue']<=0.05]
        print(res)
        res.to_csv(opts['<output>'], index=False, sep='\t')

    elif opts['--filter'] == "false":
        res.to_csv(opts['<output>'], index=False, sep='\t')
        print(res)

    else:
        print("Sorry, this is an invalid parameter ")








# all domains
# merge/split common boundary x = max(3bin,0.1 TAD Length)
# region < agrs.remote
# less complex
# zoom
# to filter the strength first
import pandas as pd
import numpy as np
from tqdm import tqdm
import argparse
import os
import warnings
warnings.filterwarnings('ignore')

# the arguments from command line
parser = argparse.ArgumentParser(description='python scriptname <-d> <-t> [options]')
parser.add_argument('-d','--diff', type=str, default = None,help="path/ the text of diffdoamin's outcome")
parser.add_argument('-t','--tad',type=str, default=None,help='path/ the other tadlist')
# parser.add_argument('-m','--method',type=str,default='BH',help='to choose the method that judge differential tads in "--diff"')
parser.add_argument('-o','--out',type=str,default=None,help='the output path')
parser.add_argument('-l','--limit',type=int,default=30000,help='the range(length of bases) to judge the common boundary')
parser.add_argument('-k','--kpercent',type=int,default=10,help='the common boundareis are within max(l*bin,k% TAD length)')
parser.add_argument('-r','--remote',type=int,default=1000000,help='the limitation of the biggest region')
parser.add_argument('-s1','--skip1',type=int,default=0,help='to skip the sth rows in "--diff" file (like 1,2,3')
parser.add_argument('-s2','--skip2',type=int,default=0,help='to skip the sth rows in the other tadlist file')
parser.add_argument('--sep1',type=str,default='\t',help="the seperator of the diffdoamin's outcome (like ',')")
parser.add_argument('--sep2',type=str,default='\t',help="the seperator of the other tadlist")
args = parser.parse_args()

# load the files

data = pd.read_table(args.diff,skiprows=range(args.skip1),sep=args.sep1)
tad = pd.read_table(args.tad,skiprows=range(args.skip2),sep=args.sep2)
# print(data)
#preprocessing
cols = data.columns
data.rename(columns={cols[0]:'chr',cols[1]:'start',cols[2]:'end'},inplace=True,errors='raise')
tad = tad.iloc[:,0:3]
tad.columns = ['chr','start','end']

data['chr'] = [str(i).replace('chr','').strip() for i in data['chr']]
tad['chr'] = [str(i).replace('chr','').strip() for i in tad['chr']]

data_diff = data.loc[data['adj_pvalue']<0.05,['chr','start','end']]
data_diff['significant'] = 1 
data_diff.reset_index(inplace=True,drop=True)

tad.sort_values(by=['chr','start','end'],inplace=True)
tad.reset_index(inplace=True,drop = True)
tad['range'] = [*map(lambda a,b:(a,b) , tad.start,tad.end)]

# preparation
chrs = list(map(str,[*range(1,24)]))+['X']
colnames = ['chr','start','end','range','type','origin','subtype','significant']
tad_ = data_main = loss = single = merge = split = multi = pd.DataFrame(columns=colnames)

tad_ = pd.concat([tad_,tad],axis=0)
tad = tad_
data_main = pd.concat([data_main,data.iloc[:,0:3]],axis=0)
data_main['significant'] = 0
data_main = pd.concat([data_main,data_diff],axis=0)
data_main.drop_duplicates(subset=['chr','start','end'],keep='last',inplace=True)
data_main['range'] = [*map(lambda a,b:(a,b) , data_main.start,data_main.end)]
data_main['origin'] = 'condition1'
data_main.sort_values(by=['chr','start','end'],inplace=True)
data_main.reset_index(inplace=True,drop=True)


def identical(boundary1,boundary2):
    # to judge the "common boundary"
    if int(boundary1) <= int(boundary2)+limit and int(boundary1) >= int(boundary2)-limit:
        return True 
    else: 
        return False

def cross(main,vise): 
    # main is the protagnist tad
    # to find the tads related to main in vise
    note=pd.DataFrame(columns=colnames)
    for i in range(vise.shape[0]):
        if (int(main[2])-limit > int(vise.iloc[i,1]) and int(main[1])+limit < int(vise.iloc[i,2]) ):
            note=pd.concat([note,pd.DataFrame(vise.iloc[i,:].values.reshape(1,-1),columns=colnames)],axis=0)
    return note

def n_of_region(outcome):
    # to count the number of regions in the dataframe 
    n_region = 0
    if len(outcome) != 0 :
        n_region = 1
        for i in range(2,len(outcome)):
            if outcome['origin'].values[i]=='conition1' and outcome['origin'].values[i-1]=='condition2':
                n_region = n_region+1
    return n_region

def n_diffdomain(outcome):
    n_diff = outcome.loc[outcome['origin']=='conition1',:].shape[0]
    return n_diff


# the 4th virsion+ bin
try:
    for c in tqdm(data_main.chr.unique()):
        temp = data_main.loc[data_main['chr']==c,:].copy()
        # print(c,temp.shape[0])
        tadlist = tad.loc[tad['chr']==c,:].copy()
        tadlist['origin'] = 'condition2'
        temp.reset_index(inplace=True,drop=True)
        tadlist.reset_index(inplace=True,drop=True)

        # filter the strength-change diffdomains and other non-significantly differentail tads with common boudaries in vise tadlist
        tad_index = []
        cross_index = []
        for i in range(temp.shape[0]): 
            # the i th TADs in the result of DiffDomain
            # to filter the TADs with common boundaries in different conditions

            # initialize the variables 
            note_tad = note_cross = pd.DataFrame(columns=colnames)
            # set the "limit" for judging the common boundaries
            limit = max(args.limit,args.kpercent*0.01*(temp['end'][i]-temp['start'][i]))
            note_tad = pd.concat([note_tad,pd.DataFrame(temp.loc[i,:].values.reshape(1,-1),columns=colnames)],axis=0)

            for k in range(tadlist.shape[0]):
                if (identical(temp.loc[i,'start'],tadlist.loc[k,'start'])) and (identical(temp.loc[i,'end'],tadlist.loc[k,'end'])) :
                    note_cross = pd.concat([note_cross,pd.DataFrame(tadlist.loc[k,:].values.reshape(1,-1),columns=colnames)],
                                           axis=0,ignore_index = True)
                    cross_index.append(k)
                    tad_index.append(i)

            n_cross = note_cross.shape[0]

            if n_cross !=0 : 
                # in case that there are TADs in temp having common boundaries but not in tadlist 

                for j in range(i+1,temp.shape[0]):
                    # to find the TADs (in the result of DiffDomain) located on the same boundaries with the i th TADs
                    if (identical(temp.loc[i,'start'],temp.loc[j,'start'])) and (identical(temp.loc[i,'end'],temp.loc[j,'end'])):
                        note_tad = pd.concat([note_tad,pd.DataFrame(temp.loc[j,:].values.reshape(1,-1),columns=colnames)],
                                             axis=0,ignore_index = True)
                        tad_index.append(i)
                        tad_index.append(j)

            note_tad.drop_duplicates(subset=['chr','start','end'],inplace=True)
            note_cross.drop_duplicates(subset=['chr','range','end'],inplace=True)
            n_tad = note_tad.shape[0]


            if n_tad ==1 and n_cross ==1 :
                note_tad['type'] = 'single'
                note_tad['subtype'] = 'strength'
                single = pd.concat([single,note_tad,note_cross],axis=0,
                                  ignore_index = True)

            elif n_tad == 1 and n_cross >=2 :
                note_tad['type'] = 'split'
                split = pd.concat([split,note_tad,note_cross],axis=0,
                                 ignore_index = True)

            elif n_tad >= 2 and n_cross ==1 :
                note_tad['type'] = 'merge'
                merge = pd.concat([merge,note_tad,note_cross],axis=0,
                                 ignore_index = True)

            elif n_tad >= 2 and n_cross >= 2 :
                if n_tad == n_cross :
                    note_tad['type'] = 'single'
                    note_tad['subtype'] = 'strength'
                    single = pd.concat([single,note_tad,note_cross],axis=0,
                                      ignore_index = True)
                else:
                    note_tad['type'] = 'complex'
                    multi = pd.concat([multi,note_tad,note_cross],axis=0,
                                     ignore_index = True)

        temp.drop(tad_index,inplace=True)
        temp.reset_index(drop=True,inplace=True)
        tadlist.drop(cross_index,inplace = True)
        tadlist.reset_index(drop=True,inplace=True)
    #         temp_sig = temp.loc[temp['significant']==1,:].copy()
    #         temp_sig.reset_index(drop = True,inplace=True)

        for i in range(temp.shape[0]):
            # to adjust the longest distance between "common boundaries"
            # to find the related TADs without common boundaries in different conditions

            limit = max(args.limit,(temp['end'][i]-temp['start'][i])*args.kpercent*0.01)
            note_cross = note_tad = pd.DataFrame(columns=colnames)

            # to find the tads in tadlist related to the significantly differential tad
            note_cross = pd.concat([note_cross,cross(temp.iloc[i,:].values,tadlist)],axis=0,
                                  ignore_index = True)

            note_tad = pd.concat([note_tad,pd.DataFrame(temp.iloc[i,:].values.reshape(1,-1),columns=colnames)],
                                 axis=0,ignore_index = True)

            n_cross = note_cross.shape[0]

            if n_cross == 0:
            # the significantly differential tad grew out of nothing
                note_tad['type'] = 'loss'
                loss = pd.concat([loss,note_tad],axis=0
                                 ,ignore_index = True)

            elif n_cross >=1:
                flag = 1
                while flag == 1 and note_tad['end'].max()-note_tad['start'].min() <= args.remote:
                    for p in range(note_cross.shape[0]):
                        # to find TADs related to the tads found in vise tadlist
                        note_tad = pd.concat([note_tad,cross(note_cross.iloc[p,:].values,temp)],axis=0,ignore_index = True)
                    for q in range(note_tad.shape[0]):
                        # to find TADs in the tadlist related to the TADs in the result of DiffDomain
                        note_cross = pd.concat([note_cross,cross(note_tad.iloc[q,:].values,tadlist)],axis=0,ignore_index = True)

                    first_tad =  note_tad.loc[note_tad.start == min(note_tad.start),:]
                    last_tad = note_tad.loc[note_tad.end == max(note_tad.end),:]
                    first_cross = note_cross.loc[note_cross.start == min(note_cross.start),:]
                    last_cross = note_cross.loc[note_cross.end == max(note_cross.end),:] 

                    thres1 = pd.concat([cross(first_tad.values[0],tadlist),cross(last_tad.values[0],tadlist)],axis=0)
                    thres2 = pd.concat([cross(first_cross.values[0],temp),cross(last_cross.values[0],temp)],axis=0)

                    if (thres1['range'].isin(note_cross['range'])).all() and thres2['range'].isin(note_tad['range']).all():
                        flag = 2


                note_tad.drop_duplicates(subset=['chr','start','end'],inplace=True)
                note_cross.drop_duplicates(subset=['chr','start','end'],inplace=True)
                note_tad.reset_index(inplace=True,drop=True)
                note_cross.reset_index(inplace=True,drop=True)
                n_tad = note_tad.shape[0]
                n_cross  = note_cross.shape[0]

                if n_tad == 1 and n_cross == 1: 
                    note_tad['type'] = 'single'
                    note_tad['subtype'] = 'zoom'
                    single = pd.concat([single,note_tad,note_cross],axis=0,ignore_index = True)

                elif n_tad == 1 and n_cross >= 2:
                    note_tad['type'] = 'split'
                    split = pd.concat([split,note_tad,note_cross],axis=0,ignore_index = True)

                elif n_tad >= 2 and n_cross ==1:
                    note_tad['type'] = 'merge'
                    merge = pd.concat([merge,note_tad,note_cross],axis=0,ignore_index = True)

                elif n_tad >=2 and n_cross >=2:
                    note_tad['type'] = 'complex'
                    multi = pd.concat([multi,note_tad,note_cross],axis=0,ignore_index = True)


except Exception as e:
    print(e)
    print('Interrupted!')
else:
    result = pd.DataFrame(columns=colnames)
    # there will be duplicates because of travelsal
    result = pd.concat([result,loss,single,merge,split,multi],axis=0,ignore_index=True)
    result.drop_duplicates(subset=['chr','start','end','origin','significant'],
                           inplace=True)

    if args.out != None:
        os.chdir(args.out)    
    filename = os.path.split(args.diff)[1]
    result.fillna('nan',inplace=True)
    result.to_csv('{}_types.txt'.format(filename),sep='\t',index=False)
    print(filename)

    # whether all of the significantly defferential tads have been selected
    len_sig = result.loc[result['significant']==1,:].shape[0]
    if len_sig == data_diff.shape[0]:
        print(f'Complete.There are {data_diff.shape[0]} reorganized TADs.')
    else:
        print('Error!')
        print(f'there are {data_diff.shape[0]} reorganized TADs,but the length of result is {len_sig}.')
        
        


    



#-*- coding : utf-8-*-
# coding:unicode_escape

from collections import defaultdict
import numpy as np
import hicstraw as straw
import TracyWidom as tw
import gzip
import pandas as pd
import os
import sys
import pandas as pd
from docopt import docopt
from multiprocessing import Pool
import warnings
warnings.filterwarnings('ignore')


def finftypes(filepath):
    """
    filepath can be stdin, normal txt, and gz file
    """
    if(filepath == 'stdin'):
        fin = sys.stdin
    elif(filepath[-3:] == '.gz'):
        fin = gzip.open(filepath)
    else:
        fin = open(filepath,'r')
    return fin


def loadtads(inpath, sep, chrnum, reso, min_nbin): # SyntaxError:non-default argument follows dafault argument
    tadb = []
    fin = finftypes(inpath)
    next(fin)
    for line in fin:
        #col = line.rstrip().split(sep)
        col = line.rstrip().split(sep)
        tadb.append(col[:3])
    fin.close()

    tadb = pd.DataFrame(tadb)
    tadb.iloc[:,1:3] = tadb.iloc[:,1:3].astype(int)

    # format the chromosomal names
    if 'chr' in tadb.iloc[0,0]:
        tadb[0] = tadb[0].str.replace('chr', '')

    if chrnum != 'ALL':
        ind = tadb[0] == chrnum
        tadb = tadb[ind] #tadb = [_ for _ in tadb if _[0]==chrnum]

    # select domain with at least min_nbin
    nbins = np.ceil(tadb[2] /reso) - np.floor(tadb[1] / reso) 
    tadb = tadb[nbins>=min_nbin] #huadm

    return tadb

def makewindow(x, reso):
    
    k = int(np.ceil(x / reso))
    
    wins = [_*reso for _ in range(k+1) ]

    return wins

def makewindow2(start, end, reso):
    if start > end :
        print('The end of TAD should be bigger than its start')
        return None
    k1 = int(np.floor(start / reso))
    k2 = int(np.ceil(end / reso))

    wins = [ _*reso for _ in range(k1, k2)]
    return wins


def compute_nbins(start, end, reso):
    domwin = makewindow2(start, end, reso)
    # create index for domwin
    nb = len(domwin)
    return nb




def contact_matrix_from_hic(chrn, start, end, reso, fhic, hicnorm):
    # handle some regions cannot be loaded by straw
    # find the bins for a domain
    domwin = makewindow2(start, end, reso)
    # create index for domwin
    nb = len(domwin)
    domwin_dict = {domwin[_]:_ for _ in range(nb)}
    # create empty matrix
    mat = np.empty(shape=(nb, nb))
    mat[:] = np.nan
    # find the edgelist from .hic
    region = "{0}:{1}:{2}".format(chrn, domwin[0], domwin[-1])
    if fhic[-4:] == '.hic':
        try:
            el = straw.straw('observed',hicnorm, fhic, region, region, 'BP', reso)
            for i in range(len(el)):
                bin0 = el[i].binX
                bin1 = el[i].binY
                k=domwin_dict[bin0]
                l=domwin_dict[bin1]
                if k == l:
                    mat[k, l] = el[i].counts
                else:
                    mat[k, l] = el[i].counts
                    mat[l, k] = el[i].counts

            return mat
        except IOError:
            print("Sorry,{%s} does't exist."% fhic )
            mat = None
            return mat

    ## This module is just written to test the .h5 sample data (chr1_50M_GM12878.h5, chr1_50M_K562.h5)
    elif fhic[-3:] == '.h5':
        import h5py
        hf = h5py.File(fhic,'r')
        regions = "{0}:{1}:{2}".format(chrn, start, end)
        el = hf.get(regions)
        el = np.array(el) # transform to numpy format

        for i in range(len(el[2])):
            bin0 = el[0][i]
            bin1 = el[1][i]
            k=domwin_dict[bin0]
            l=domwin_dict[bin1]
            if k == l:
                mat[k, l] = el[2][i]
            else:
                mat[k, l] = el[2][i]
                mat[l, k] = el[2][i]

        return mat

    # load a sparse matrix with three columns
    else:
        data = pd.read_table(fhic,sep='\t')
        el =[[],[],[]]
        el[0] = data[data.columns[0]].tolist()
        el[1] = data[data.columns[1]].tolist()
        el[2] = data[data.columns[2]].tolist()
        
        for i in range(len(el[2])):
            bin0 = el[0][i]
            bin1 = el[1][i]
            k=domwin_dict[bin0]
            l=domwin_dict[bin1]
            if k == l:
                mat[k, l] = el[2][i]
            else:
                mat[k, l] = el[2][i]
                mat[l, k] = el[2][i]

        return mat


def extractKdiagonalCsrMatrix(spsCsrMat):
    nbin = spsCsrMat.shape[0]
    nonZeroIndex = spsCsrMat.nonzero()
    contactByDistance = defaultdict(list)
    for _ in range(len(nonZeroIndex[0])):
        rowIndex = nonZeroIndex[0][_]
        colIndex = nonZeroIndex[1][_]
        if colIndex >= rowIndex:
            dist = colIndex - rowIndex
            contactByDistance[dist].append(spsCsrMat[rowIndex, colIndex])

    numNonzero = []
    median = []
    for k, v in contactByDistance.items():
        numNonzero.append(len(v))
        median.append(np.nanmedian(v))


    #print numNonzero[:10], numNonzero[-10:]
    #print median[:10], median[-10:]

    return contactByDistance

def normDiffbyMeanSD(D):
    # log transformation
    D = np.log(D)
    contactByDistanceDiff = extractKdiagonalCsrMatrix(D)
    # imputation of nan and inf
    # part0: get the median and maximum for each off-diagonal
    a, b = defaultdict(list), defaultdict(list)
    m, sd = defaultdict(float), defaultdict(float)
    for k, val in contactByDistanceDiff.items():
        indnan = np.isnan(val)
        indinf = np.isinf(val)
        if np.any(indnan) and np.any(indinf):
            val = np.array(val)
            ind = np.logical_or(indnan, indinf)
            val1 = val[np.logical_not(ind)]
            a[k] = np.median(val1)
            b[k] = np.max(val1)
            m[k] = np.mean(val1)
            sd[k] = np.std(val1)
        elif np.any(indnan) and not np.any(indinf):
            val = np.array(val)
            val1 = val[np.logical_not(indnan)]
            a[k] = np.median(val1)
            m[k] = np.mean(val1)
            sd[k] = np.std(val1)
        elif not np.any(indnan) and np.any(indinf):
            val = np.array(val)
            val1 = val[np.logical_not(indinf)]
            b[k] = np.max(val1)
            m[k] = np.mean(val1)
            sd[k] = np.std(val1)
        else:
            m[k] = np.mean(val)
            sd[k] = np.std(val)

    #part1: impute nan and inf
    indnan = np.isnan(D)
    indnan.astype(int)
    indr, indc = np.nonzero(indnan)
    for _ in range(len(indr)):
        k = abs(indr[_] - indc[_])
        D[indr[_], indc[_]] = a[k]

    indinf = np.isinf(D)
    indr, indc = np.nonzero(indinf)
    for _ in range(len(indr)):
        k = abs(indr[_] - indc[_])
        D[indr[_], indc[_]] = b[k]

    # subtract mean and dividing by sd
    for k, v in sd.items():
        if np.isnan(v) or v == 0:
            sd[k] = 1

    nd = D.shape[0]
    for i in range(nd):
        for j in range(nd):
            k = abs(i-j)
            D[i, j] = (D[i,j] - m[k]) / sd[k]


    return D

def twtest_formula(Mat):
    nd = Mat.shape[0]
    # normalize it to a Wigner matrix
    Mat = Mat / np.sqrt(nd)
    try:
        w, v = np.linalg.eigh(Mat)
        #U, s, V = np.linalg.svd(Mat, full_matrices=True)
        #w = np.abs(w)
        #w.sort()
        #lambdan = w[-1]
        lambdan = np.amax(np.abs(w))
        # normalize by sqrt(n)
        lambdan = np.power(nd, 2.0/3) * (lambdan -2)

        twdist = tw.TracyWidom(beta=1)
        p = 1 - twdist.cdf(lambdan)
    except:
        nd = np.nan
        lambdan = np.nan
        p = np.nan

    return nd, lambdan, p

def visualization(chrn, start, end, reso, hicnorm, fhic0, fhic1):

        mat0 = contact_matrix_from_hic(chrn, start, end, reso, fhic0, hicnorm)
        mat1 = contact_matrix_from_hic(chrn, start, end, reso, fhic1, hicnorm)
        return mat0,mat1


def comp2domins_by_twtest(chrn, start, end, reso, hicnorm, fhic0, fhic1, min_nbin, f):
        mat0 = contact_matrix_from_hic(chrn, start, end, reso, fhic0, hicnorm)
        mat1 = contact_matrix_from_hic(chrn, start, end, reso, fhic1, hicnorm)

        if not mat0 is None and not mat1 is None:
            # rlmove rows that have more than half np.nan
            nbins = compute_nbins(start, end, reso)
            ind0 = np.sum(np.isnan(mat0), axis=0) < nbins * (1-float(f))
            ind1 = np.sum(np.isnan(mat1), axis=0) < nbins * (1-float(f))
            ind = ind0 & ind1
            if ind.sum() >= min_nbin:
                indarray = np.array(ind)
                mat0rmna = mat0[indarray, :]
                mat0rmna = mat0rmna[:, indarray]
                mat1rmna = mat1[indarray, :]
                mat1rmna = mat1rmna[:, indarray]

                # compute the differnece matrix
                Diffmat = mat0rmna / mat1rmna
                #print Diffmat
                ind1 = np.sum(np.isnan(Diffmat), axis=0)
                #print ind1

                Diffmatnorm = normDiffbyMeanSD(D=Diffmat)
                #print Diffmatnorm

                result = twtest_formula(Diffmatnorm)

                domname = 'chr%s:%s-%s' % (chrn, start, end)
                result = [chrn, start, end, domname, result[1], result[2], result[0]]
            else:
                domname = 'chr%s:%s-%s' % (chrn, start, end)
                print('The length of this TAD is too small at this resolution to be calculated !')
                result = [chrn, start, end, domname, np.nan, np.nan, np.nan]
        else:
            domname = 'chr%s:%s-%s' % (chrn, start, end)
            result = [chrn, start, end, domname, np.nan, np.nan, np.nan]
            print('The matrix is too spase at this resolution to be calculated !')

        return result






import setuptools

with open('README.rst','r') as fh:
    long_description = fh.read()
    
setuptools.setup(
    name = 'diffDomain-py3',
    version = '0.1.2',
    author = 'Dechao Tian',
    author_email = 'tiandch@mail.sysu.edu.cn',
    description = 'DiffDomain can test the significant difference of TADs.',
    long_description = long_description,
    url = 'https://github.com/Tian-Dechao/diffDomain',
    packages = setuptools.find_packages(),
    install_requires=['hic-straw==1.3.1',
                     'TracyWidom',
                      'pandas',
                      'numpy',
                      'docopt',
                      'statsmodels',
                      'seaborn',
                      'matplotlib',
                      'h5py'
                     ],
    classifiers = [
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]

)
